from typing import Type, Union, Literal, Optional, Dict, List, Tuple, Set, Annotated, Any
from pydantic import BaseModel, Field, HttpUrl
from pydantic import model_validator
from mindor.dsl.schema.action import HttpClientActionConfig
from .common import ComponentType, CommonComponentConfig

class HttpClientComponentConfig(CommonComponentConfig):
    type: Literal[ComponentType.HTTP_CLIENT]
    base_url: Optional[HttpUrl] = None
    headers: Optional[Dict[str, Any]] = Field(default_factory=dict)
    actions: Optional[Dict[str, HttpClientActionConfig]] = Field(default_factory=dict)
    
    @model_validator(mode="before")
    def inflate_single_action(cls, values):
        if "actions" not in values:
            action_keys = set(HttpClientActionConfig.model_fields.keys())
            if any(k in values for k in action_keys):
                values["actions"] = { "__default__": { k: values.pop(k) for k in action_keys if k in values } }
        return values

    @model_validator(mode="after")
    def validate_baseurl_with_actions(self):
        for action_id, action in self.actions.items():
            if action.path and not self.base_url:
                raise ValueError(f"Action '{action_id}' uses 'path' but 'base_url' is not set in the component.")
        return self

    @model_validator(mode="after")
    def validate_baseurl_with_completion(self):
        # for action_id, action in self.actions.items():
        #     if action.completion and action.completion.path and not self.base_url:
        #         raise ValueError(f"Completion for action '{action_id}' uses 'path' but 'base_url' is not set in the component.")
        return self
